
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include <iostream.h>
#include <ctype.h>

#include "base/ObjId.h"
#include "runtime/LogComment.h"
#include "client/ForeignWorld.h"


int ForeignWorld::TheWss = 0;

ForeignWorld::ForeignWorld() {
}

void ForeignWorld::configure(const String &aName, istream &is) {
	Assert(!id());
	id(UniqId::Create());
	theName = aName;

	Comment(7) << "loading URL trace '" << theName << "'" << endl;

	Size memSize = 0;
	char line[16*1024];
	while (is.getline(line, sizeof(line))) {
		// delete comments
		if (char *comment = strchr(line, '#'))
			*comment = '\0';

		// trim white space
		const char *urlB = line;
		const char *urlE = urlB + strlen(urlB);
		while (urlB < urlE && isspace(*urlB))
			++urlB;
		while (urlB < urlE && isspace(urlE[-1]))
			--urlE;

		if (urlB < urlE) {
			if (strncasecmp(urlB, "http://", 7) != 0) {
				Comment(1) 
					<< "error: all trace URLs must be of "
					<< "http://host/path format; " << endl
					<< "\tfound: " << urlB << endl
					<< endc;
			}
			String *s = new String;
			s->append(urlB, urlE - urlB);
			theUrls.append(s);
			memSize += urlE - urlB;     // content
			memSize += SizeOf(String);  // overhead
			memSize += SizeOf(String*); // overhead
		}
	}

	if (theUrls.count())
		Comment(5) << "fyi: loaded ";
	else
		Comment(1) << "warning: empty ";

	Comment << "trace '" << theName << "': "
		<< "URL count: " << theUrls.count() 
		<< ", memory usage: " << memSize
		<< endc;
}

bool ForeignWorld::canProduce() const {
	return ObjWorld::canProduce() && theSize < theUrls.count();
}

void ForeignWorld::repeat(ObjId &oid, ObjSelector *sel) {
	static const String tag("foreign");
	oid.foreignUrl(tag); // hack: let underlying levels know the oid is foreign
	ObjWorld::repeat(oid, sel);
	Assert(oid.name() > 0);
	oid.foreignUrl(*theUrls[oid.name() % theUrls.count()]); // now set real URL
}

void ForeignWorld::produce(ObjId &oid, RndGen &rng) {
	ObjWorld::produce(oid, rng);
	incWss(TheWss);
	Assert(oid.name() > 0);
	oid.foreignUrl(*theUrls[oid.name() % theUrls.count()]);
}

