
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include "xstd/h/iostream.h"
#include "xstd/h/strstream.h"

#include "base/RndPermut.h"
#include "runtime/IOBuf.h"
#include "csm/ContentDbase.h"

#include "csm/ContentCfg.h"
#include "csm/InjectIter.h"
#include "csm/CdbBodyIter.h"

#include "xstd/gadgets.h"


CdbBodyIter::CdbBodyIter(): theCdb(0), theInjector(0), 
	thePos(-1), theEntryOff(-1) {
}

void CdbBodyIter::reset() {
	Assert(!theInjector);
	BodyIter::reset();
	theCdb = 0;
	thePos = -1;
	theEntryOff = -1;
}

void CdbBodyIter::cdb(ContentDbase *aCdb) {
	Assert(!theCdb && aCdb);
	theCdb = aCdb;
}

void CdbBodyIter::injector(InjectIter *anInjector) {
	Assert(!theInjector && anInjector);
	theInjector = anInjector;
}

void CdbBodyIter::startPos(int aPos) {
	Assert(thePos == -1 && aPos >= 0);
	thePos = aPos;
	theEntryOff = 0;
}

void CdbBodyIter::configure(WrBuf *aBuf, Size aContSize) {
	BodyIter::configure(aBuf, aContSize);
	Assert(theCdb && theCdb->count());
	Assert(thePos >= 0);
	Assert(theEntryOff == 0);
	if (theInjector)
		theInjector->start(this);
}

void CdbBodyIter::stop() {
	if (theInjector) {
		theInjector->creator()->putInjector(theInjector);
		theInjector = 0;
	}
}

void CdbBodyIter::pourBody() {
	CdbEntryPrnOpt opt;
	opt.buf = theBuf;
	opt.injector = theInjector;
	while (theBuiltSize < theContSize && !theBuf->full()) {
		const CdbEntry *e = theCdb->entry(thePos);
		const Size mark = theBuf->contSize();
		// decision really changes only once per entry
		if (e->minSize() - theEntryOff <= sizeLeft()) {
			opt.sizeMax = sizeLeft();
			opt.entryOff = theEntryOff;
			e->pour(opt);

			theEntryOff += theBuf->contSize() - mark;
			Should(theEntryOff <= e->size());

			if (e->size() <= theEntryOff) {
				theEntryOff = 0;
				thePos++;
				thePos %= theCdb->count();
			}
		} else {
			// current entry does not fit, stuff with random data
			Size rndOff = theBuf->normalRndOffset(offSeed(), theBuiltSize);
			theBuf->appendRndUpTo(rndOff, sizeLeft());
		}
		theBuiltSize += theBuf->contSize() - mark;
		//cerr << here << "appended: " << sz << "; " << theBuiltSize << " <= " << theContSize << endl;
	}
}
