
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef PG_CONTENT_DATABASE_ENTRIES_H
#define PG_CONTENT_DATABASE_ENTRIES_H

#include "csm/ContentDbase.h"

class StrIdentifier;

enum { cdbeNone = 0, cdbeText, cdbeTag, cdbeBlob, cdbeComment, cdbeEnd };

// opaque, unbreakable piece of data
class CdbeBlob: public CdbEntry {
	public:
		virtual int type() const { return cdbeBlob; }
		virtual Size size() const { return theImage.len(); }

		virtual void image(const String &image);

		virtual OBStream &store(OBStream &ol) const;
		virtual IBStream &load(IBStream &il);

		virtual ostream &print(ostream &os) const;
		virtual void pour(CdbEntryPrnOpt &) const;

	public:
		String theImage;
};


// breakable piece of text, can take injections
class CdbeText: public CdbeBlob {
	public:
		virtual int type() const { return cdbeText; }
		virtual Size minSize() const { return Size(1); }

		virtual void pour(CdbEntryPrnOpt &opt) const;
};

// unbreakable text inside <!-- -->
// need to add inside-breakability and inside-injection functionalities
class CdbeComment: public CdbeBlob {
	public:
		virtual int type() const { return cdbeComment; }
};


// unbreakable tag
class CdbeTag: public CdbeBlob {
	public:
		virtual int type() const { return cdbeTag; }

		virtual void image(const String &image);

	protected:
		static const String *AttrValReplacement(const String &key);
		static int RegReplacement(const String &key, const String &ctype);

		String calcCheckSum(const char *str, int len) const;

	protected:
		static StrIdentifier *TheReplIdentifier;
		static Array<String*> TheReplacements;
};
  
#endif
