
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef PG_BLOB_DATABASE_H
#define PG_BLOB_DATABASE_H

#include "xml/XmlNodes.h"
#include "loganalyzers/InfoScope.h"
#include "loganalyzers/BlobIdx.h"

class ReportBlob;

// a collection of ReportBlobs
class BlobDb {
	public:
		typedef InfoScope Scope;

	public:
		static String Key(const String &name, const Scope &scope);
		static String KeySuffix(const Scope &scope);

	public:
		BlobDb();

		const ReportBlob *has(const String &key);
		const ReportBlob *add(const ReportBlob &b);
		BlobDb &operator <<(const ReportBlob &b) { add(b); return *this; }

		const XmlNodes &blobs() const { return theBlobs; }

		// never fail
		void link(const String &oldString, const String &newString);
		const ReportBlob &get(const String &key);
		const XmlNode &include(const String &key);
		XmlNode &ptr(const String &key, const XmlNodes &context);

		ostream &print(ostream &os, const String &pfx) const;

	protected:
		const ReportBlob *find(const String &key) const;

	protected:
		XmlNodes theBlobs;
		BlobIdx theBlobIdx;
		XmlNodes theIncludes;
		XmlNodes thePtrs;
		XmlNodes theErrors;
};

inline
String operator +(const String &name, const InfoScope &scope) {
        return BlobDb::Key(name, scope);
}

inline
String operator +(const char *name, const InfoScope &scope) {
        return String(name) + scope;
}


#endif
