
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef PG_LOAD_STATS_EXTRACTOR_H
#define PG_LOAD_STATS_EXTRACTOR_H

#include "xstd/String.h"
#include "base/StatIntvlRec.h"

// an algorithm of extracting a particular level statistics out of
// interval stats record
class LoadStex {
	public:
		typedef double (StatIntvlRec::*StatPtr)() const;

	public:
		LoadStex(const String &aKey, const String &aName, StatPtr aRate, StatPtr aBwidth):
			theKey(aKey), theName(aName), 
			theRateStats(aRate), theBwidthStats(aBwidth) {}

		const String &key() const { return theKey; } // precise, for machine use
		const String &name() const { return theName; } // imprecise, human-readable

		double rate(const StatIntvlRec &rec) const { return (rec.*theRateStats)(); }
		double bwidth(const StatIntvlRec &rec) const { return (rec.*theBwidthStats)(); }

	protected:
		String theKey;
		String theName;
		StatPtr theRateStats;
		StatPtr theBwidthStats;
};

#endif
