
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include <xstd/h/os_std.h>
#include <fstream.h>

#include "xml/XmlAttr.h"
#include "xml/XmlText.h"
#include "xml/XmlParagraph.h"
#include "xml/XmlTable.h"
#include "loganalyzers/ReportBlob.h"
#include "loganalyzers/BlobDb.h"
#include "loganalyzers/RepOpts.h"
#include "loganalyzers/PhaseInfo.h"
#include "loganalyzers/ReportFigure.h"

String ReportFigure::TheBaseDir = ".";


ReportFigure::ReportFigure(): thePhase(0), theCtrlFile(0) {
}

ReportFigure::~ReportFigure() {
	delete theCtrlFile;
}

void ReportFigure::configure(const String &aKey, const String &aTitle) {
	theKey = aKey;
	theTitle = aTitle;
	theBaseName = TheBaseDir + '/' + theKey;
	thePlotFname = theBaseName + ".png";
	theCtrlFname = theBaseName + ".gp";
}

const ReportBlob &ReportFigure::plot(BlobDb &db) {
	const int pointCount = createCtrlFile();
	const bool success = pointCount > 0 && plotCtrlFile();
	destroyCtrlFile();

	ReportBlob blob(theKey);

	if (success) {
		XmlTable table;
		table << XmlAttr::Int("border", 1);

		XmlTableRec r1;
		r1 << XmlTableHeading(title());
		table << r1;

		XmlTableRec r2;
		XmlTableCell cell;
		XmlTag img("img");
		img << XmlAttr("src", thePlotFname) << XmlAttr("alt", blob.key());
		cell << img;
		r2 << cell;
		
		table << r2;
		blob << table;
		return *db.add(blob);
	} else 
	if (pointCount == 0) {
		XmlTextTag<XmlParagraph> warning;
		warning.buf() << "Figure '" << title() << "' not available: "
			<< "no datapoints to plot.";
		blob << warning;
	} else {
		XmlTextTag<XmlParagraph> error;
		error.buf() << "Figure '" << title() << "' not available: "
			<< "something went wrong while generating the graph.";
		blob << error;
	}

	return *db.add(blob);
}

int ReportFigure::createCtrlFile() {
	theCtrlFile = new ofstream(theCtrlFname.cstr());
	if (!theCtrlFile || !*theCtrlFile) {
		clog << "error: cannot create " << theCtrlFname << ": " << Error::Last() << endl;
		return false;
	}

	setCtrlOptions();
	return *theCtrlFile ? 0 : -1;
}

bool ReportFigure::plotCtrlFile() {
	const String cmd = TheRepOpts.thePlotter + ' ' + theCtrlFname +
		" > " + thePlotFname;
	if (::system(cmd.cstr()) != 0) {
		clog << "error: cannot plot " << theCtrlFname 
			<< "; consult error messages above, if any" << endl;
		clog << "plot file was:" << endl;
		system((String("cat ") + theCtrlFname).cstr());
		return false;
	}
	return true;
}

bool ReportFigure::destroyCtrlFile() {
	delete theCtrlFile;
	theCtrlFile = 0;
	return unlink(theCtrlFname.cstr()) == 0;
}

void ReportFigure::setCtrlOptions() {
	*theCtrlFile << "set term png color small" << endl;
	*theCtrlFile << "set output '" << thePlotFname << "'" << endl;

	*theCtrlFile << "set title ''" << endl;
	*theCtrlFile << "set grid" << endl;
	*theCtrlFile << "set linestyle 1 lt 3" << endl;
	*theCtrlFile << "set linestyle 2 lt 1" << endl;
	*theCtrlFile << "set linestyle 3 lt 2" << endl;
	*theCtrlFile << "set linestyle 4 lt 4" << endl;
}
